#include <msp/fs/utils.h>
#include "component.h"
#include "externaltask.h"
#include "microsofttools.h"
#include "msvcarchiver.h"
#include "objectfile.h"
#include "sourcepackage.h"
#include "staticlibrary.h"

using namespace std;
using namespace Msp;

MsvcArchiver::MsvcArchiver(Builder &b, const Architecture &a, const MicrosoftTools &m):
	Tool(b, a, "AR"),
	ms_tools(m)
{
	input_suffixes.push_back(".o");
	processing_unit = COMPONENT;
	set_command((ms_tools.get_vc_bin_dir()/"lib.exe").str(), false);
}

Target *MsvcArchiver::create_target(const list<Target *> &sources, const string &)
{
	if(sources.empty())
		throw invalid_argument("MsvcArchiver::create_target");

	list<ObjectFile *> objs;
	for(list<Target *>::const_iterator i=sources.begin(); i!=sources.end(); ++i)
	{
		if(ObjectFile *obj = dynamic_cast<ObjectFile *>(*i))
			objs.push_back(obj);
		else
			throw invalid_argument("MsvcArchiver::create_target");
	}

	const Component &comp = *objs.front()->get_component();
	StaticLibrary *lib = new StaticLibrary(builder, comp, objs);
	lib->set_tool(*this);
	return lib;
}

string MsvcArchiver::create_build_signature(const BuildInfo &) const
{
	return FS::basename(executable->get_path());
}

Task *MsvcArchiver::run(const Target &target) const
{
	const StaticLibrary &lib = dynamic_cast<const StaticLibrary &>(target);
	const Component &comp = *lib.get_component();

	vector<string> argv;
	argv.push_back(executable->get_path().str());
	argv.push_back("/NOLOGO");

	FS::Path work_dir = comp.get_package().get_source_directory();

	argv.push_back("/OUT:"+relative(lib.get_path(), work_dir).str());

	const Target::Dependencies &deps = lib.get_dependencies();
	for(Target::Dependencies::const_iterator i=deps.begin(); i!=deps.end(); ++i)
		if(ObjectFile *obj = dynamic_cast<ObjectFile *>(*i))
			argv.push_back(relative(obj->get_path(), work_dir).str());

	return new ExternalTask(argv, work_dir);
}
